# Localization

In order to handle the complex international differences in number and date formats `react-widgets` relies on third party
parsing and formatting libraries via an integration layer of "localizers". `react-widgets` comes with
built-in localizers for __Globalize.js__, __Moment.js__ and a simple number localizer, but you can easily write
your own for whatever library you are using.

Localization sensitive widgets have `format` props that passed directly to your chosen localizer. The type and shape
of these format props is determined by the api of the backing I18n strategy. Moment.js, for instance,
uses string based date formats like: `MMM dd YYYY` whereas the newer Globalize.js may take an object like:
`{ skeleton: GyMMMd }`. Regardless of the localizer, formats always accept `function` values for doing on-the-fly custom
formatting.

<div className='alert alert-info'>
  Migrating from v2 and happy with the old default? `npm i --save globalize@0.1.x` and use the globalize localizer.
</div>

### Globalize.js (recommended) <small>date, number</small>

Globalize can be a bit of a challenge to set up correctly so please consult the [globalize docs](https://github.com/jquery/globalize#getting-started) for a lot of info on setting up
Globalize and integrating into lots of different app environments.

The globalize localizer handles both __number__ and __date__ localization so there is no need for any additional
localizers.

```js
var Globalize = require('globalize')
var globalizeLocalizer = require('../lib/localizers/globalize')

Globalize.load(/* load the date and number modules */)
Globalize.locale('en-GB')

globalizeLocalizer(Globalize);
```
Or
```html
  <script src='node_modules/react-widgets/dist/react-widgets.js'></script>
  <script src='node_modules/react-widgets/dist/react-widgets-globalize.js'></script>
```

Globalize [formats](https://github.com/jquery/globalize/blob/master/doc/api/date/date-formatter.md#parameters) can
be either `string`s, or `object`s. Date `string` formats are automatically considered `raw` patterns,
and are equivalent to `globalize.formatDate(date, { raw: format })`. Number formats accept any of
the [Globalize number options](https://github.com/jquery/globalize/blob/master/doc/api/number/number-formatter.md#parameters).
In addition you can specify _currencies_ by adding the `currency` key like: `{ currency: "USD", style: 'decimal' }`.

```js
  <DateTimePicker format='mmm YYY'/>
  <DateTimePicker format={{ date: 'full' }}/>

  <NumberPicker format={{ style: 'percent', maximumFractionDigits: 2 }}/>
  <NumberPicker format={{ currency: 'USD', style: "accounting" }}/>
```

### Moment.js <small>date</small>

Again see the official [Moment docs](http://momentjs.com/) for information on integrating Moment into your build
pipeline effectively.

Moment only provides __date__ localization, if you also need Number localization consider
the __simple-number__ localizer below, or Globalize.js.

```js
var Moment = require('moment')
var momentLocalizer = require('../lib/localizers/moment')

momentLocalizer(Moment);
```
Or
```html
  <script src='node_modules/react-widgets/dist/react-widgets.js'></script>
  <script src='node_modules/react-widgets/dist/react-widgets-moment.js'></script>
```

Moment [format](http://momentjs.com/docs/#/displaying/format/) props accept `string`s

```js
 <DateTimePicker format='mmm YYY'/>
```

### Simple Number <small>number</small>

The `simple-number` localizer provides a minimal number formatting and parsing strategy. Its best when you don't need
robust locale support for currencies, and numbers;

```js
var numberLocalizer = require('../lib/localizers/simple-number')

numberLocalizer();
```
Or
```html
  <script src='node_modules/react-widgets/dist/react-widgets.js'></script>
  <script src='node_modules/react-widgets/dist/react-widgets-simple-number.js'></script>
```

Check out the documentation for [format-number-with-string](https://www.npmjs.com/package/format-number-with-string) for
a complete guide to its format syntax.

```js
  <NumberPicker format='-$#,###.00'/>
```

## Creating a Localizer

Creating a localizer is as easy as providing `react-widgets` an localizer options object.
Localizers must provide `parse()` and `format()` functions as well as provide default values for all the
required formats the widgets need.

Formats can be whatever type your localization strategy requires (strings, objects, etc), however functions are
always valid. The default formats, for example, can be strings or functions.
If you wanted to use the built-in `Intl` api's for formatting, formats might be an options object to
pass to `Intl.DateTimeFormat()`. Function formats are called automatically by the localizer with the `value`,
the `culture` string and the localizer instance.

```js
var localizer = {

  formats: {
    day: 'DD',
    month: 'mmm',
    // we always pass a function for more advanced formats such as returning a year 'range'
    // to represent a decade e.g "2000 - 2009". Notice the localizer instance is the third argument,
    // which can be used to format or parse as needed.
    decade: (date, culture, localizer) => {
      return localizer.format(date, 'YYYY') + ' - ' + localizer.format(lastYearOfDecade(date), 'YYYY')
    }
  },

  parse(value, format, cultureStr){
    return parsedDate
  },

  format(value, format, cultureStr){
    return formattedDateString
  }
}

ReactWidgets.setDateLocalizer(localizer)
```

## Localizer Api

### `DateLocalizer`
An Object implementing the following api.

```
{
  propType: PropType?,
  formats: Object,
  firstOfWeek: Function,
  parse: Function,
  format: Function
}
```

#### required formats
_Localizers must provide default values for each required format._

- `default`: the default date diaplay format, generally a "long" format showing both date and time
- `date`: A date only format
- `time`: A time only format
- `header`: The heading of the Calendar month view, contextuals the current month, e.g. "Jan 2014"
- `footer`: The Calendar footer format, for displaying Today's date
- `dayOfMonth`: The day of the month
- `month`: Month name, used in the Year view of the Calendar
- `year`: year format, used in the Decade view of the Calendar
- `decade`: a decade format, used in the Century view of the Calendar, eg. "2010 - 2019"
- `century`: A century format, used the in the Calendar heading


#### `propType` (optional)
A React PropType that is used to validate the Date formats

#### `parse`
Convert a locale formatted string to a JavaScript Date object.

```
function(
  value: String,
  format: String|Object,
  culture: String?
) -> Date?
```

#### `format`
Convert a Date object to a locale specific string

```
function(
  value: Date?,
  format: String|Object,
  culture: String?
) -> String
```

#### `firstOfWeek`

Return the locale specific first day of the week from 0 (Sunday) to 6 (Saturday).

```
function(
  culture: String?
) -> number (0 -6)
```

### `NumberLocalizer`
An Object implementing the following api.
```
{
  propType: PropType?,
  formats: Object,
  parse: Function,
  format: Function,
  precision: Function
}
```

#### required formats
_Localizers must provide default values for each required format._

- `default` The number picker display format.

#### `propType` (optional)
A React PropType that is used to validate the number formats.

#### `parse`
Convert a locale specific string to a JavaScript Number.
```
function(
 value: String,
 culture: String?
) -> Date?

```

#### `format`

Convert a Number to a locale specific string.

```
function(
 value: Number?,
 format: String|Object,
 culture: String?
) -> String
```


#### `precision`

Return the decimal precision for a given format or culture. Necessary for dealing with the quirks of floating point math.

```
function(
 format: String|Object,
 culture: String?
) -> Number
```
